package cologne.eck.image_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main view of image pea. 
 */

/*
 * Frame to display images, decrypted in ImageControl
 * (this is not a daughter class of ContentUtilities!)
 * 
 */



import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;

import javax.accessibility.AccessibleContext;
import javax.activation.MimetypesFileTypeMap;
import javax.imageio.ImageIO;
import javax.swing.*;

import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.HelpMenu;
import cologne.eck.all_peas.gui.MultiFileSelectionDialog;
import cologne.eck.all_peas.gui.PeaMenuBar;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.SingleFileSelectionDialog;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WriteResources;
import cologne.eck.tools.Zeroizer;


@SuppressWarnings("serial")
public
final class LockFrameImage extends JFrame 
	implements ActionListener, ImageObserver, WindowListener {	
	//, AdjustmentListener {
	
	/**
	 * All file names of the image folder
	 */
	private static ArrayList <String> fileList;
	
	/**
	 * The index of the current image
	 */
	int index = 0; 
	
	// the loaded image as an array of bytes
	private static byte[] imageBytes;
	
	// the image to show
	private static BufferedImage image = null;		
	// scroll pane containing the image
	private JScrollPane scrollPane = null;
	// vertical ScrollBar
	private JScrollBar verticalBar = null;//scrollPane.getVerticalScrollBar();
	// the label containing the scroll pane
	private JLabel picLabel = null;
	// label to show error and info messages 
	private JLabel messageLabel = null;


	// zoom value for width: additional width
	private static int zoomWidth = 0;
	// zoom value for height: additional height
	private static int zoomHeight = 0;
	
	// resized default width from ImageControl
	private int imgWidth = 0;
	// resized default height from ImageControl
	private int imgHeight = 0;

	
	private LockFrameImage(Point location) {
		
		PswDialogView.setUI(false);
		
	    this.setIconImage(PswDialogView.getImage());
		
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
		this.setContentPane(contentPane);		
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		menuPanel.setMaximumSize(new Dimension(600, 30));
		menuPanel.setMinimumSize(new Dimension(300, 30));
		
		PeaMenuBar menuBar = new PeaMenuBar();
		//menuBar.setBorder(new EtchedBorder() );
		
		JMenu menu = new JMenu();
		menu.setText(PeaProperties.getBundle().getString("file"));//"Menu");
		menu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getBundle().getString("file_mnemonic")).getKeyCode());
		menuBar.add(menu);
		
		JMenuItem addNewItem = new JMenuItem();
		addNewItem.setText(PeaProperties.getBundle().getString("add_new_file"));
		addNewItem.setActionCommand("addNewFile");
		//encryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//addNewItem.setMnemonic(KeyEvent.VK_E);
		addNewItem.addActionListener(this);
		menu.add(addNewItem);
		
		JMenuItem encryptItem = new JMenuItem();
		encryptItem.setText(PeaProperties.getBundle().getString("close_encrypted"));
		encryptItem.setActionCommand("encrypted");
		//encryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//encryptItem.setMnemonic(KeyEvent.VK_E);
		encryptItem.addActionListener(this);
		menu.add(encryptItem);

		JMenuItem unencryptItem = new JMenuItem();
		unencryptItem.setText(PeaProperties.getBundle().getString("close_unencrypted"));
		unencryptItem.setActionCommand("unencrypted");
		//unencryptItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//unencryptItem.setMnemonic(KeyEvent.VK_U);
		unencryptItem.addActionListener(this);
		menu.add(unencryptItem);

		JMenuItem pswChangeItem = new JMenuItem();
		pswChangeItem.setText(PeaProperties.getBundle().getString("change_password"));//"change password");
		pswChangeItem.setActionCommand("changePassword");
		//pswChangeItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//pswChangeItem.setMnemonic(KeyEvent.VK_P);
		pswChangeItem.addActionListener(this);
		menu.add(pswChangeItem);

		JMenuItem gotoItem = new JMenuItem();
		gotoItem.setText(PeaProperties.getBundle().getString("go_to"));
		gotoItem.setActionCommand("goto");
		//gotoItem.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//gotoItem.setMnemonic(KeyEvent.VK_G);
		gotoItem.addActionListener(this);
		menu.add(gotoItem);
		

		JMenu helpMenu = new HelpMenu();
		menuBar.add(helpMenu);

		menuPanel.add(menuBar);
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		
		JPanel messagePanel = new JPanel();
		messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.X_AXIS));
//		messagePanel.setPreferredSize(new Dimension (300, 30));
		messagePanel.setMaximumSize(new Dimension(
				(int)PswDialogView.getScreenwidth()-100, 50));
		messagePanel.setMinimumSize(new Dimension (300, 30));
		
		JButton previousButton = new JButton();//previous");
		Image previousImage = null;
		URL url = this.getClass().getResource("resources/zurueck.png");
		if (url == null) {
			try{
				previousImage = new ImageIcon(getClass().getClassLoader().getResource("resources/zurueck.png")).getImage();
			} catch (Exception e) {
				//System.out.println("image not found " + e.getMessage() + ", " + e.toString());
			}
		} else {			
			previousImage = new ImageIcon(url).getImage();
		}
		if (previousImage != null) {
			previousButton.setIcon(new ImageIcon(previousImage));
		} else { // set characters
			previousButton.setText(" <-");
			previousButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 15));
		}
		previousButton.setMaximumSize(new Dimension(75,25));
		previousButton.setMargin(new Insets(1,1,1,1));
		previousButton.addActionListener(this);
		previousButton.setActionCommand( "previous");
		
		messagePanel.add(previousButton);
		messagePanel.add(Box.createHorizontalStrut(25));
		

		messageLabel = new JLabel(" Image PEA " + PeaControl.getDialog().getVersion());
		//messageLabel.setPreferredSize(new Dimension(300, 30));
		messageLabel.setMinimumSize(new Dimension(100, 30));
		messageLabel.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth() - 250, 30));
		messagePanel.add(messageLabel);
		messagePanel.add(Box.createHorizontalGlue());		
		
		JButton nextButton = new JButton();
		Image nextImage = null;
		url = this.getClass().getResource("resources/weiter.png");
		if (url == null) {
			try{
				nextImage = new ImageIcon(getClass().getClassLoader().getResource("resources/weiter.png")).getImage();
			} catch (Exception e) {
				//System.out.println("image not found " + e.getMessage() + ", " + e.toString());
			}
		} else {			
			nextImage = new ImageIcon(url).getImage();
		}

		if (nextImage != null) {
			nextButton.setIcon(new ImageIcon(nextImage));
		} else { // set characters
			nextButton.setText("-> ");
			nextButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 15));
		}		
		//JButton nextButton = new JButton("next");
		nextButton.setMaximumSize(new Dimension(75,25));
		nextButton.setMinimumSize(new Dimension(75,25));
		nextButton.setMargin(new Insets(1,1,1,1));
		nextButton.addActionListener(this);
		nextButton.setActionCommand( "next");
		messagePanel.add(nextButton);
		
		contentPane.add(messagePanel);
		
		boolean success = setNewImage(imageBytes);
		Window win;
		if (PswDialogView.getView().isVisible()){
			win = PswDialogView.getView();
		} else {
			win = (JFrame) PeaControl.getDialog().getLockFrame();
		}
		if (success == false) {
			
			String decryptionMessage = "";
		
			if(CipherStuff.getErrorMessage() != null){
				decryptionMessage = PeaProperties.getBundle().getString(CipherStuff.getErrorMessage());
			}
			CipherStuff.setErrorMessage(null);

			if (fileList == null || fileList.size() == 0) {
				System.err.println("There a no files to load...");
				success = false;
			} else {
				String fileNameString = "";
				if (fileList.get(index) != null) {
					fileNameString = fileList.get(index);
					JOptionPane.showMessageDialog(win, 
							PeaProperties.getBundle().getString("cant_load_image")
							+ "\n   " + fileNameString
							+ "\n" + decryptionMessage, 
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				}
				int len = fileList.size();
				if (len <= 1) {
					JOptionPane.showMessageDialog(win, 
							PeaProperties.getBundle().getString("cant_load_any_image"), 
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
					windowClosingCommands();
				} else {
					index++;
					while (success == false && index < len){
						success = loadOtherImage(index);	
						index++;
					}
				}
			}
		}
		if (success == false) {
			JOptionPane.showMessageDialog(win, 
					PeaProperties.getBundle().getString("cant_load_any_image")
					+ "\n  " + PeaProperties.getBundle().getString("remove_and_try_other"), 
					PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
			windowClosingCommands();
		}
				
		// set maximum size, depending on screen size 
		scrollPane.setMaximumSize(new Dimension (
						(int)PswDialogView.getScreenwidth() - 100, 
						(int) PswDialogView.getScreenheight() - 300 ));// think of buttons
		// avoid page scroll for PDFs
		verticalBar = scrollPane.getVerticalScrollBar();
		verticalBar.setValue(5);

		// register scroll to begin and to end to render next pages
//		verticalBar.addAdjustmentListener(this);
		contentPane.add(scrollPane);
		
		JPanel zoomPanel = new JPanel();
		zoomPanel.setMaximumSize(new Dimension(400, 30));
		zoomPanel.setMinimumSize(new Dimension(200, 30));
		
		JButton zoomInButton = new JButton ("+");
		zoomInButton.setActionCommand("zoomIn");
		zoomInButton.addActionListener(this);
		zoomPanel.add(zoomInButton);	
		
		JButton zoomOutButton = new JButton ("-");
		zoomOutButton.setActionCommand("zoomOut");
		zoomOutButton.addActionListener(this);
		zoomPanel.add(zoomOutButton);	
		
		zoomPanel.add(zoomInButton);
		zoomPanel.add(zoomOutButton);
		
		contentPane.add(zoomPanel);		

		this.setLocation(location);
		SwingUtilities.updateComponentTreeUI(this);
		
		pack();
		
		// pack or setMaximumSize/MinimumSize depending on screen size and image size
		if (this.getPreferredSize().getWidth() > PswDialogView.getScreenwidth() - 100
			|| this.getPreferredSize().getHeight() > PswDialogView.getScreenheight() - 200) {
			
			this.setMaximumSize(
					new Dimension (
							(int)PswDialogView.getScreenwidth() - 100,
							(int)PswDialogView.getScreenheight() - 200));
			int w = ImageControl.getImageWidth();
			if (w > PswDialogView.getScreenwidth() - 100){
				w = (int) (PswDialogView.getScreenwidth() - 100);
			}
			int h = ImageControl.getImageHeight();
			if (h > PswDialogView.getScreenheight() - 200){
				h = (int) (PswDialogView.getScreenheight() - 200);
			}
			// is is the shown size if the screen is large enough
			this.setMinimumSize(new Dimension (w, h));			
		} 
	}
	/**
	 * Returns an already instantiated instance 
	 * 
	 * @return 
	 */
	public final static LockFrameImage getInstance(Point loc) {		
		
		
		if (PeaControl.getDialog().getLockFrame() == null) {
			return new LockFrameImage(loc);
		} else {
			return (LockFrameImage) PeaControl.getDialog().getLockFrame();
		}
	}
	 // just to test the view	 
/*	public static void main(String[] args) {

		PswDialogView.setUI(false);
		

//		File testFile = new File("resources/Phahlbaudorf.JPG");
		File testFile = new File("resources/images/drop.png");
		fileList = new ArrayList<String>();
		String imageFolder = "resources/images";
		File folder = new File(imageFolder);
		File[] files = folder.listFiles();
		int len = files.length;
		for(int i = 0; i < len; i++) {
			fileList.add(files[i].getAbsolutePath());
			System.out.println(files[i].getAbsolutePath());
		}
		LockFrameImage.setImageBytes(ReadResources.readExternFile(testFile.getAbsolutePath()));

		LockFrameImage frame = new LockFrameImage(new Point(100,100));
		frame.messageLabel.setText(testFile.getAbsolutePath());
		
		frame.setVisible(true);
	} */
	
	/**
	 * The method resizes an image to keep the ratio of width and height
	 * to avoid distortions of the image. The result width is never smaller
	 * than the default width / 4. 
	 * 
	 * @param image		the image to resize
	 * @param width		the width of the frame to display the image
	 * @param height	the height of the frame to display the image
	 * 
	 * @return			resulting width ad height, stored in an array:
	 * 					result[0] is the new width, result[1] the new height to display
	 */
	private final int[] resizeImage(BufferedImage image, int width, int height){
		// keep ratio of image width and height: resize the image 

		int[] result = new int[2];
		int displayedWidth = width;
		int displayedHeight = height;			
		int imageWidth = image.getWidth();
		int imageHeight = image.getHeight();
		double op = 0; // factor to resize		
		//System.out.println("image w: " + image.getWidth() + ", h: " + image.getHeight()
		//		+ "\n" + "default w: " + displayedWidth + ", h: " + displayedHeight);
		 
		 if (imageWidth < width) {
			 		
			 op = (double)width / imageWidth;

			 if (imageHeight * op > height) {
				 
				 op = (double)height / imageHeight;
				 // do not resize smaller than half of default width: 
				 // this is normally a rendered PDF
				 if ( (imageWidth * op) < (width / 4) ) {
					 op = 1;//imageWidth / (double)width;
				 }
			 }
			 
			 displayedWidth = (int) (imageWidth * op);
			 displayedHeight = (int) (imageHeight * op);
		 } else {
			 op = imageWidth / (double)width;

			 if ( (imageHeight / op) > height) {
				 op = imageHeight / (double)height;
				 // do not resize smaller than half of default width: 
				 // this is normally a rendered PDF
				 if ( (imageWidth / op) < (width / 4) ) {
					 op = 1;//imageWidth / (double)width;
				 }
			 }
			 
			 displayedWidth = (int) (imageWidth / op);
			 displayedHeight = (int) (imageHeight / op);
		 }	
		 result[0] = displayedWidth;
		 result[1] = displayedHeight;
		 //System.out.println("result w: " + result[0] + ", h: " + result[1]);
		 return result;
	}

	/**
	 * Zoom in or out: Resize the image, display the image and set the scroll bars 
	 * 
	 * @param in	if true: zoom in, otherwise zoom out
	 */
	private final void zoom(boolean in){
		
		  int previouseV = scrollPane.getVerticalScrollBar().getValue();
		  int previouseH = scrollPane.getHorizontalScrollBar().getValue();
		
		if (in == true) {			

			zoomWidth += imgWidth/2;
			zoomHeight += imgHeight/2;
			
		} else {
			
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;	
			
			if (imgWidth + zoomWidth <= 0 || imgHeight + zoomHeight <= 0){
				//System.out.println("Zoom out: invalid size values for image.");				
				zoomWidth += imgWidth/2;
				zoomHeight += imgHeight/2;
				// avoid IllegalArgumentException: do not paint the image
				return;
			}
		}
		try {
			// new compressed/resized image
			BufferedImage  compressedImage = new BufferedImage(
					imgWidth + zoomWidth, imgHeight + zoomHeight,
                  BufferedImage.TYPE_INT_ARGB);
            Graphics2D g = compressedImage.createGraphics();
            g.drawImage(image, 0, 0, imgWidth + zoomWidth, imgHeight + zoomHeight, this);              
            g.dispose();		
		  
             
            picLabel.setIcon(new ImageIcon(compressedImage));
               
      		if (in == true) {      	
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV + imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH + imgWidth/4));                
      		} else {
                scrollPane.getVerticalScrollBar().setValue((int) (previouseV - imgHeight/4));
                scrollPane.getHorizontalScrollBar().setValue((int) (previouseH - imgWidth/4));
      		}
		} catch (IllegalArgumentException e){  // width and height are < 0

			if (PeaProperties.getWorkingMode().equals("-t")) { // test mode
				System.out.println(e.toString() + ", " + e.getMessage());
			}		 
			// reset the zoom
			if (in == true) {
				 zoomWidth -= ImageControl.getImageWidth() / 2;
				 zoomHeight -= ImageControl.getImageHeight() / 2;
			} else {
				 zoomWidth += ImageControl.getImageWidth() / 2;
				 zoomHeight += ImageControl.getImageHeight() / 2;		
			}
		} catch (OutOfMemoryError ex) { // ... if the image is too large
			if (PeaProperties.getWorkingMode().equals("-t")) { // test mode
				System.err.println(ex.toString() + " - " + ex.getMessage());
			  
				System.err.println("total memory: " + Runtime.getRuntime().totalMemory());
				System.err.println("free memory: " + Runtime.getRuntime().freeMemory());
				// used space: image width * image height * 8 (TYPE_INT_ARGB) 
				System.err.println("used space for image: " + ((imgWidth + zoomWidth) * (imgHeight + zoomHeight) * 8));
			}		  
			zoomWidth -= imgWidth/2;
			zoomHeight -= imgHeight/2;
		}
	}
	/**
	 * Checks if a file is a valid image that can be shown
	 * by ImagePEA. The file must exist and must have read access,
	 * otherwise the check fails. The file is at least checked by 
	 * ImageIO, but before - depending on parameters - checked by
	 * it's MIME type ("image") and it's extension (PNG, GIF, BMP, JEPG, JPG) 
	 * 
	 * @param file				the file to check
	 * @param checkExtension	whether to check the extension of the file or not
	 * @param checkMimeType		whether to check the MIME type of the file or not
	 * 
	 * @return	true if the file is a image that can be shown by the Image PEA
	 */
	public static boolean checkImage(File file, boolean checkExtension, boolean checkMimeType) {
		
		if ( ! file.isFile()) {
			return false;
		}
		
		String fileName = file.getAbsolutePath();
		
		// check MIME type:
		if (checkMimeType == true) {
			 String mimetype= new MimetypesFileTypeMap().getContentType(file);
		        String type = mimetype.split("/")[0];
		        if( ! type.toLowerCase().equals("image")) {
		        	return false;
		        }
		}
		
		// check extension: 
		if (checkExtension == true) {
			String extension = fileName.substring(fileName.lastIndexOf('.') + 1,
				fileName.length()).toUpperCase();
			//System.out.println("extension: " + extension);
			if (! (extension.equals("PNG") || extension.equals("GIF")
					|| extension.equals("BMP") || extension.equals("JPG")
					|| extension.equals("JPEG")) ){ 
				return false;
			}
		}
		
		// check by ImageIO:
		try{
			BufferedImage bi = ImageIO.read(file);	
			if (bi == null) {
				return false;
			}
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	/**
	 * Set the image to show. This class holds a copy of the image as an array of bytes
	 * 
	 * @param input		the image as an array of bytes
	 */
	public final static void setImageBytes( byte[] input){//, String fileName ) {
		if (input == null) {
			new UnexpectedValueException("input", "byte[]", "is  null").printDescription();
		}
		imageBytes = new byte[input.length];
		System.arraycopy(input,  0,  imageBytes,  0,  input.length);
	}
	
	/**
	 * Draws an image in the panel, shows an error
	 * message if fails and packs. 
	 * 
	 * @param imageBytes	the image to draw as byte array
	 * 
	 * @return	true, if the image was drawn successfully, 
	 * 			false otherwise (image should be removed then from list)
	 */
	private final boolean setNewImage(byte[] imageBytes) { //, String fileName) {
		
		if (imageBytes == null) { // was set to null before
			if (PeaProperties.getWorkingMode().equals("-t")) { // test mode	
				new UnexpectedValueException("imageBytes", "byte[]", "is null").printDescription();
			}
			return false;
		}

		// get image from file name:		
		try {					
			image = ImageIO.read(new ByteArrayInputStream(imageBytes));

		} catch (IOException e) {			
			System.err.println("LockFrameImage: " + e.toString() + " - " + e.getMessage());
			if (PeaProperties.getWorkingMode().equals("-t")) { // test mode				
				e.printStackTrace();
			} 
		}
		Window win;
		if (PswDialogView.getView().isVisible()){
			win = PswDialogView.getView();
		} else {
			win = (JFrame) PeaControl.getDialog().getLockFrame();
		}
		if (image == null) { // invalid file
			JOptionPane.showMessageDialog(win, 
					PeaProperties.getBundle().getString("invalid_image_file") + "\n" //"Invalid or unsuitable image file: \n"                      
							+ fileList.get(index),
//							+ ImageControl.getEncryptedFileName(), 
					      PeaProperties.getBundle().getString("error"),                                
					   JOptionPane.ERROR_MESSAGE);   
			return false;
		}
		// resize the image
		int[] newSize = resizeImage(image, ImageControl.getImageWidth(), ImageControl.getImageHeight());
		imgWidth = newSize[0];
		imgHeight = newSize[1];	
		
		// check if image is larger than default values for width and height: 
		if (image.getWidth() > imgWidth || image.getHeight() > imgHeight){
			// compress the image:
			BufferedImage  resizedImage = new BufferedImage(imgWidth + zoomWidth, imgHeight + zoomHeight,
	        BufferedImage.TYPE_INT_ARGB);
	        Graphics2D g = resizedImage.createGraphics();
	        g.drawImage(image, 0, 0, imgWidth + zoomWidth, imgHeight + zoomHeight, this);              
	        g.dispose();	

	        if (picLabel== null) {
	        	picLabel= new JLabel();
	        }
			picLabel.removeAll();
			picLabel.setIcon(new ImageIcon(resizedImage));
//			scrollPane = new JScrollPane(picLabel);
			if (scrollPane == null) {
				scrollPane = new JScrollPane(picLabel);
			}
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
		} else {
			// do not compress
	        if (picLabel== null) {
	        	picLabel= new JLabel();
	        }
			picLabel.removeAll();
			picLabel.setIcon(new ImageIcon(image));

			if (scrollPane == null) {
				scrollPane = new JScrollPane(picLabel);
			}
			scrollPane.setPreferredSize(new Dimension(imgWidth, imgHeight));
			imgWidth = image.getWidth();
			imgHeight = image.getHeight();
		}
		//currentFileName = fileName;
		
		this.pack();
		return true;
	}
	
	public void setMessage(String _message) {

		messageLabel.setText(_message);
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();

		if (command.equals("unencrypted")) {
			
			MultiFileSelectionDialog dialog = new MultiFileSelectionDialog(
					this, this.getLocationOnScreen(), 
					PeaProperties.getBundle().getString("select_files_remain_unencrypted"),
					fileList);
			if (dialog.getSelectedFiles() == null) {				
				return;
			}
			ArrayList<String> unencryptedList = dialog.getSelectedFiles();
			dialog.dispose();
			
			if (unencryptedList == null || unencryptedList.size() == 0) {
				return;
			}
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());
				messageLabel.setText(PeaProperties.getBundle().getString("decryption") + " " + PeaProperties.getBundle().getString("in_progress"));
				// the current displayed file:
				// the index will change because elements will be removed
				String currentFileName = fileList.get(index);
				
				int len = unencryptedList.size();
				int fileListLen = fileList.size();
				boolean closeCurrentImage = false;
				for (int i = 0; i < len; i++) {
					if ( ! new File(unencryptedList.get(i)).isFile()) {
						// directory
						// set null in fileList to mark as invalid
						for (int j = 0; j < fileListLen; j++) {
							if (fileList.get(j).equals(unencryptedList.get(i))) {
								fileList.set(j,  null);
							}
						}
						continue;
					}
						
					if (unencryptedList.get(i) == null) {
						System.err.println("File name is null at index " + i);
						continue;
					}
						
					if (unencryptedList.get(i).equals(currentFileName)) { // current shown image:
						closeCurrentImage = true;
						// close at least
						continue;
					}
					//read cipherBytes, decrypt and store
					byte[] cipherBytes = ReadResources.readExternFile(unencryptedList.get(i));
					byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
					if (plainBytes == null) {
						System.err.println("Decryption failed at index " + i
								+ " for file name " + unencryptedList.get(i));
						JOptionPane.showMessageDialog((Window)PeaControl.getDialog().getLockFrame(), 
								PeaProperties.getBundle().getString("decryption_failed")
								+ "\n" + unencryptedList.get(i)
								+ PeaProperties.getBundle().getString(CipherStuff.getErrorMessage()), 
								PeaProperties.getBundle().getString("error"), 
								JOptionPane.ERROR_MESSAGE);
						CipherStuff.setErrorMessage(null);
						// do not store in path file
						// set null in fileList to mark as invalid
						for (int j = 0; j < fileListLen; j++) {
							if (fileList.get(j).equals(unencryptedList.get(i))) {
								fileList.set(j,  null);
							}
						}
						
						PeaControl.getAvailableFileNames().remove(unencryptedList.get(i));
						continue;
					}
					WriteResources.write(plainBytes, unencryptedList.get(i), null );
				}
				if (closeCurrentImage == true ) {
					//read cipherBytes, decrypt and store
					byte[] cipherBytes = ReadResources.readExternFile(currentFileName);
					byte[] plainBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );
					if (plainBytes == null) {
						System.err.println("Decryption failed for " + currentFileName);
						JOptionPane.showMessageDialog((Window) PeaControl.getDialog().getLockFrame(), 
								PeaProperties.getBundle().getString("decryption_failed")
								+ "\n" + currentFileName
								+ PeaProperties.getBundle().getString(CipherStuff.getErrorMessage()),
								PeaProperties.getBundle().getString("error"), 
								JOptionPane.ERROR_MESSAGE);
						CipherStuff.setErrorMessage(null);
						// do not store in path file
						PeaControl.getAvailableFileNames().remove(currentFileName);
						for (int j = 0; j < fileListLen; j++) {
							if (fileList.get(j).equals(currentFileName)) {
								fileList.set(j,  null);
							}
						}
						windowClosingCommands();
					}
						
					WriteResources.write(plainBytes, currentFileName, null );						
					// give message:
					JOptionPane.showMessageDialog(this,
							PeaProperties.getBundle().getString("current_file_closed"));
					// close program: 
					Zeroizer.zero(imageBytes);
					windowClosingCommands();
					System.exit(0);
				}

			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
				messageLabel.setText("");
			}
			// only close program if current file was selected
			//System.exit(0);
			
		} else if (command.equals("encrypted")){
			// file was not modified, just loaded unencrypted in RAM
			PathFileManager.pathFileCheck(fileList);
			Zeroizer.zero(imageBytes);
			windowClosingCommands();
			
		} else if (command.equals("addNewFile")){
			
			PeaControl.getDialog().initializeNewFiles();
			
			
		} else if (command.equals("changePassword")){
		
			// show a list of files
			MultiFileSelectionDialog dialog = new MultiFileSelectionDialog(
					this, this.getLocationOnScreen(), 
					PeaProperties.getBundle().getString("Select_file_for_password_change"),
					fileList);
			
			// the list of the selected files
			ArrayList<String> unencryptedList = dialog.getSelectedFiles();
			// check if current file is included:
			if ( ! (unencryptedList.contains(fileList.get(index))) ){
				JOptionPane.showMessageDialog(this, 
						PeaProperties.getBundle().getString("must_contain_current_file") +":\n" + fileList.get(index), 
						PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				return;
			} 
			dialog.dispose();			

			// Note: Wait cursor is set in ContentUtilities.changePassword
			String[] arrayToChangePassword = null;
			if (unencryptedList != null && unencryptedList.size() > 0) {
				// create array from the list:
				arrayToChangePassword = new String[unencryptedList.size()];
				arrayToChangePassword = unencryptedList.toArray(arrayToChangePassword);
			}
			ContentUtilities.changePassword( arrayToChangePassword, this);	// file names to change

		} else if (command.equals("zoomIn")){
			zoom(true);
		} else if (command.equals("zoomOut")){
			zoom(false);
		} else if (command.equals("about")) {
			PswDialogView.showAboutDialog("Image Lock PEA", PeaProperties.getBundle().getString("supported_formats") + ": png, jpg, gif, wbmp", this);
			
		} else if (command.equals("infos")){			
			
			PswDialogView.showInfoDialog(this);
		} else if (command.equals("previous")){				
			
			int newIndex = 0;
			if (index == 0) { // begin...
				// show last image:
				newIndex = fileList.size() - 1;
			} else {
				// show previous image:
				newIndex = index - 1;
			}
			boolean success = false;
			String rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
			
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());			
			
				while (newIndex != index ) { // index is only updated for success
					// end of loop if no valid image is found
					if (newIndex < 0) {
						newIndex = fileList.size() - 1;						
					}
	
					if (fileList.get(newIndex) == null) {
						// was marked as invalid
						newIndex--;
						continue;
					} else {
						// try to load previous image:
						rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
						success = loadOtherImage(newIndex);
					}					
					if (success == true) {
						this.setCursor(CursorManager.getDefaultCursor());
						return;
					} else {
						String errorString = CipherStuff.getErrorMessage();
						try {
							errorString = PeaProperties.getBundle().getString(errorString);
						} catch (Exception e) {
							//new UnexpectedValueException("errorString", "String", "is not key in language bundle: " + errorString).printDescription();
						}
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getBundle().getString("cant_load_image")
								+ ": \n" + rememberedFileName
								+ "\n" + errorString,
								PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						// reset wait cursor because of modal dialog
						this.setCursor(CursorManager.getWaitCursor());
						CipherStuff.setErrorMessage(null);

						// try image before previous image
						newIndex--;
						if (newIndex < 0) { // begin...
							// show last image:
							newIndex = fileList.size() - 1;
						}
					}
				}
				if (success == false) {
					
					// try to load current image again:
					rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
					success = loadOtherImage(index);
					if (success = false) {
						System.err.println("Can't load " + rememberedFileName);
						String errorString = CipherStuff.getErrorMessage();
						try {
							errorString = PeaProperties.getBundle().getString(errorString);
						} catch (Exception e) {
							new UnexpectedValueException("errorString", "String", "is not key in language bundle: " + errorString).printDescription();
						}
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getBundle().getString("cant_load_any_image")
								+ "\n" + errorString,
								PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						windowClosingCommands();
					}
				}
			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
			}
			
		} else if (command.equals("next")){		
			
			// register loading and decryption sucess of new image
			boolean success = false;
			// index of the image to show next
			int newIndex = 0;
			if (index + 1 >= fileList.size()) { // end...
		    	// set first image:
		    	newIndex = 0;
			} else {
				// set next image:
				newIndex = index + 1;
			}
			String rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
			try {
				// set wait cursor
				this.setCursor(CursorManager.getWaitCursor());
						
				while (newIndex != index) { // index is only updated for success
					// end of loop if no valid image is found
					if (newIndex >= fileList.size()) { // end...
				    	// set first image:
				    	newIndex = 0;
				    	if (newIndex == index) {
				    		// no other valid image found
							JOptionPane.showMessageDialog(this, 
									PeaProperties.getBundle().getString("cant_load_other_image"), 
									PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
							// reset wait cursor because of modal dialog
							this.setCursor(CursorManager.getWaitCursor());
				    	}
					}
			
					if (fileList.get(newIndex) == null) {
						// image is marked as invalid...
						newIndex++;
						continue;
					} else {					
						if (new File(fileList.get(newIndex)).isDirectory()) {
							//fileList.remove(newIndex);
							newIndex++;
							continue;
						}
						// try to load next image:
						rememberedFileName = fileList.get(newIndex);// is set to null if loadOtherImage fails 
						success = loadOtherImage(newIndex);
					}	
					
					if (success == true) {
						this.setCursor(CursorManager.getDefaultCursor());
						return;
					} else {
						String errorString = CipherStuff.getErrorMessage();
						try {
							errorString = PeaProperties.getBundle().getString(CipherStuff.getErrorMessage());
						} catch (Exception e) {
							//new UnexpectedValueException("errorString", "String", "is not key in language bundle: " + errorString).printDescription();
						}
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getBundle().getString("cant_load_image") + ": \n" + rememberedFileName
								+ "\n" + errorString,
								PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						CipherStuff.setErrorMessage(null);
					}
					// reset wait cursor because of modal dialog
					this.setCursor(CursorManager.getWaitCursor());
					
					// try image after next image
					newIndex++;
			
				} // end of loop
				if (success == false) {
					// try to load current image again:
					success = loadOtherImage(index);
					if (success = false) {
						JOptionPane.showMessageDialog(this, 
								PeaProperties.getBundle().getString("cant_load_any_image")
								+ "\n" + PeaProperties.getBundle().getString(CipherStuff.getErrorMessage()),
								PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
						windowClosingCommands();
					}
				}
			} finally {
				this.setCursor(CursorManager.getDefaultCursor());
			}
			
		}  else if (command.equals("goto")) {
			SingleFileSelectionDialog dialog = new SingleFileSelectionDialog(
					((Window)PeaControl.getDialog().getLockFrame()), 
					((Window) PeaControl.getDialog().getLockFrame()).getLocationOnScreen(), 
					PeaProperties.getBundle().getString("select_image_file"), 
					fileList);
			String fileName = SingleFileSelectionDialog.getSelectedFileName();
			dialog.dispose();
			String rememberedFileName = fileName;// is set to null if loadOtherImage fails 
			boolean success = loadOtherImage(fileList.indexOf(fileName));
			if (success == false) {
				String error = CipherStuff.getErrorMessage();
				try { 
					error = PeaProperties.getBundle().getString(error);
				} catch (Exception e) {
					System.err.println("Can't get language String for " + error);
				}
				JOptionPane.showMessageDialog(this, 
						PeaProperties.getBundle().getString("cant_load_image") + ": \n" + rememberedFileName
						+ "\n"+ error, 
						PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
				CipherStuff.setErrorMessage(null);
				// load last image:
				success = loadOtherImage(index);
				if (success == false) {
					JOptionPane.showMessageDialog(this, 
							PeaProperties.getBundle().getString("cant_load_any_image"), 
							PeaProperties.getBundle().getString("error"), JOptionPane.ERROR_MESSAGE);
					windowClosingCommands();
				}
			} else {
				messageLabel.setText(fileName);				
			}
		}
	}

	/**
	 * Loads an image from the list, sets the image 
	 * to null if it fails
	 * 
	 * @param newIndex	the index of the image to load
	 * 
	 * @return	true: success, 
	 * 			false otherwise (image is set to null in this case)
	 */
	private boolean loadOtherImage(int newIndex) {
		
		if (newIndex < 0 || newIndex >= fileList.size()) {
			new UnexpectedValueException("newIndex", "int", 
					"is not in range 0 - " + fileList.size() + ": " + newIndex).printDescription();
			return false;
		}
		
		if (fileList.get(newIndex) == null) {
			return false;
		}
		
		boolean success = true;
		byte[] newImageBytes = null;

		if ( new File(fileList.get(newIndex)).isFile()) {
			try {
				byte[] cipherBytes = null;
				if (fileList.get(newIndex).equals(PeaProperties.getBundle().getString("image_inside_jar_archive"))){
					cipherBytes = ReadResources.getResourceFromJAR("resources/text.lock");
				} else {
					cipherBytes = ReadResources.readExternFile(fileList.get(newIndex));
				}
				//currentFileName = fileList.get(fileList.size() - 1);
				newImageBytes = CipherStuff.getInstance().decrypt(cipherBytes, null, true );

			} catch (Exception e) {
				// show the previous image:
				System.err.println(e.toString() + ", " + e.getMessage());
				success = false;
				// set element null to indicate error
				fileList.set(newIndex, null);	
				//fileList.remove(newIndex);
			}
		} else {
			// mark as invalid
			fileList.set(newIndex, null);
			//fileList.remove(newIndex);
		}
		if (newImageBytes != null) {
			// set newImageByte as current image
			imageBytes = newImageBytes;
			// load new image:
			boolean loadSuccess = setNewImage(imageBytes);
			if (loadSuccess == false){
				fileList.set(newIndex, null);	
				//fileList.remove(newIndex);
				return false;
			}
			messageLabel.setText(fileList.get(newIndex));
			// set new index as current index
			index = newIndex;
		} else {
			System.err.println("An error occured");
			fileList.set(newIndex, null);	
			success = false;
		}		
		return success;
	}
	
	@Override
	public void windowClosing(WindowEvent arg0) { // by X-Button
		windowClosingCommands();
	}
	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) {}
	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}
	
	protected void windowClosingCommands() {
		// ask to remember files:
		try {
			if (fileList != null && fileList.size() > 0) {
				
				// first remove null elements to avoid NullPointerExc.
				// use iterator to avoid ConcurrentModificationException
				Iterator<String> iti = fileList.iterator();
				while (iti.hasNext()) {
					String s = iti.next();
					if (s == null) {
						iti.remove();
					}
				}
				PathFileManager.pathFileCheck(fileList);
			}
			// zeroize current image
			if (imageBytes != null) {
				Zeroizer.zero(imageBytes);
			}
			// ********************* later version... **************	
			//PdfToPngConverter.closeDocument();
			// set location and dimension
			new PropertyHandler().updateAllPropValues();
		} catch (Exception e) {
			System.err.println(e.getLocalizedMessage() + "\n" + e.toString());
			e.printStackTrace();
		} finally {		
			CipherStuff.getInstance().getSessionKeyCrypt().clearKeys();
			System.exit(0);
		}
	}


	/**
	 * Set an new list of image file names. Already existing list will be
	 * overwritten
	 * 
	 * @param newList	the new list
	 */
	public static void setFileList(ArrayList <String> newList) {
		
		fileList = newList;
	}
	/**
	 * Get the list of image file names
	 * 
	 * @return	the list of image file names
	 */
	public static ArrayList<String> getFileListXXX() {
		return fileList;
	}
	/**
	 * Add a file name to the list of image file names
	 * 
	 * @param newFileName	the file name of the image file
	 */
	public static void addFileToList(String newFileName) {
		if ( new File(newFileName).isFile()) {
			if (fileList == null) {
				fileList = new ArrayList<String>();
			}
			if ( ! fileList.contains(newFileName)) {
				fileList.add(newFileName);
			} else {
				System.err.println("list already contains file " + newFileName 
						+ "\n(in LockFrameImage.addFileToList)");
			}
		}
	}

    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameImage();
        }
        return accessibleContext;
    }

    protected class AccessibleLockFrameImage extends AccessibleJFrame {
        //Inherit everything, override nothing.
    }
}