package cologne.eck.tools;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import javax.swing.JFrame;

import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;


public class PropertyHandler {

	// only file name, path inside jar is always /, not File.separator
	private static final String propFileName = "pea.properties";//"resources" + File.separator + 
	
	//private 	String result = "";
	private InputStream inputStream;
	private OutputStream outputStream;
	
	/**
	 * Update all values in pes.properties file
	 * that are not PEA specific
	 * 
	 * @return	error message or null
	 */
	public String updateAllPropValues() {

		String errorMessage = null;
		try {
			//==================================================
			// set properties for all PEAs
			Properties prop = new Properties();
			File file = new File(System.getProperty("user.dir") + File.separator 
					+ "resources" + File.separator + propFileName);
			if (! file.exists()) {
				// create directory
				file.getParentFile().mkdirs();
				// create file
				file.createNewFile();
			} 
			outputStream = new FileOutputStream(file);

			// Color ===============================
			Color peaColor = PeaProperties.getPeaColor();
			prop.setProperty("pea_color_r", "" + peaColor.getRed());
			prop.setProperty("pea_color_g", "" + peaColor.getGreen());
			prop.setProperty("pea_color_b", "" + peaColor.getBlue());
			// Font size ================================
			prop.setProperty("font_size", "" + PeaProperties.getFontSize());
			// language =================================
//			prop.setProperty("lang", PeaProperties.getLanguage());
			// dimension and location ========================
			JFrame frame = (JFrame) PeaControl.getDialog().getLockFrame();
			if (frame == null) {
				new UnexpectedValueException("frame", "JFrame", "is null").printDescription();
			} else {
				prop.setProperty("frame_x", "" + frame.getLocationOnScreen().getX());
				prop.setProperty("frame_y", "" + frame.getLocationOnScreen().getY());
				prop.setProperty("frame_width", "" + frame.getSize().getWidth());
				prop.setProperty("frame_height", "" + frame.getSize().getHeight());
			}
			if (outputStream != null) {
				prop.store(outputStream, "Properties for all PEAs");

			} else {
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("outputStream", "OutputStream", "file not found: " + "resources" + File.separator + propFileName).printDescription();
				errorMessage = "file not found: " + "resources" + File.separator + propFileName;
			}
			try {
				outputStream.close();
			} catch (IOException e) {
				System.err.println("IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
			//==========================================================
			// set PEA specific properties: 
			PeaControl peaControl = PeaControl.getDialog();
			
			// There are no specific properties for Image Lock PEA and File Lock PEA:
			String fileType = PeaProperties.getFileType();
			if (fileType.equals("file")) { // File Lock PEA
				return errorMessage;
			} else if (fileType.equals("image")) { // Image Lock PEA
				return  errorMessage;			
			}
			
			prop = new Properties();
			file = new File(System.getProperty("user.dir") + File.separator 
					+ "resources" + File.separator + PeaControl.getPeaPropFileName());
			if (! file.exists()) {
				// create directory
				file.getParentFile().mkdirs();
				// create file
				file.createNewFile();
			}
			outputStream = new FileOutputStream(file);
			peaControl.updatePeaPropValues(prop);//, outputStream);
			if (outputStream != null) {
				peaControl.updatePeaPropValues(prop);
				prop.store(outputStream, "Properties for specific PEA");
			} else {
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("outputStream", "OutputStream", "file not found: " 
						+ "resources" + File.separator + propFileName).printDescription();
				errorMessage = "file not found: " + "resources" + File.separator + propFileName;
			}
/*			if (fileType.equals("text file")) { // Notebook PEA
				
			} else if (fileType.equals("file")) { // Notebook PEA
				
			} else if (fileType.equals("image")) { // Notebook PEA
				
			} else if (fileType.equals("calendar")) { // Notebook PEA
				CalendarSetting.setDefaultFileName(prop.getProperty("default_file_name"));
			} else {
				new UnexpectedValueException("fileType", "String", "does not match expected values: " + fileType).printDescription();
				errorMessage += "\nFileType does not match expected values: " + fileType;
			} */

		} catch (IOException e) {
			System.err.println("IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("Exception: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				if (outputStream != null) {
					outputStream.close();
				}
			} catch (IOException e) {
				System.err.println("IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;

	}
	
	public String setAllPropValues() {// throws IOException {
		String errorMessage = null;
		 
		try {
			//==================================================
			// set properties for all PEAs
			Properties prop = new Properties();
 
			File file = new File(System.getProperty("user.dir") + File.separator + "resources" + File.separator + propFileName);
			if (file.exists()) {
				// external file in resources
				inputStream = new FileInputStream(file);
			} else {
				// inside jar: default
				System.out.println("Load default properties...");
				inputStream = getClass().getResourceAsStream( "/resources/" +  propFileName);
			} 
			if (inputStream != null) {
				prop.load(inputStream);
			} else {// should not happen... 
				//throw new FileNotFoundException("property file '" + propFileName + "' not found in the classpath");
				new UnexpectedValueException("inputStream", "InputStream", "file not found: " + "/resources/" + propFileName).printDescription();				
				errorMessage = "file not found: " + propFileName;		
				return errorMessage;
			}
			
			// should not happen:
			if (prop.getProperty("pea_color_r") == null
					&& prop.getProperty("font_size") == null
					&& prop.getProperty("frame_x") == null) {
				System.err.println("Can't read properties from file " + "resources" + File.separator + propFileName);
				return errorMessage + ("Can't read properties from file " + "resources" + File.separator + propFileName);
			}
			// Color ===============================
			int r, g, b;
			String rString = prop.getProperty("pea_color_r");
			try {
				r = Integer.parseInt(rString);
			} catch (Exception e) {
				r = PeaProperties.getPeaColor().getRed();
				new UnexpectedValueException("rString", "String", "is not digit: " + rString).printDescription();
			}
			String gString = prop.getProperty("pea_color_g");
			try {
				g = Integer.parseInt(gString);
			} catch (Exception e) {
				g = PeaProperties.getPeaColor().getGreen();
				new UnexpectedValueException("gString", "String", "is not digit: " + gString).printDescription();
			}
			String bString = prop.getProperty("pea_color_b");
			try {
				b = Integer.parseInt(bString);
			} catch (Exception e) {
				b = PeaProperties.getPeaColor().getBlue();
				new UnexpectedValueException("bString", "String", "is not digit: " + bString).printDescription();
			}
			PeaProperties.setPeaColor(new Color(r, g, b));
			// Font size ================================
			String fontSize = prop.getProperty("font_size");
			try {
				PeaProperties.setFontSize(Integer.parseInt(fontSize));
			} catch (Exception e) {
				new UnexpectedValueException("fontSize", "String", "is not digit: " + fontSize).printDescription();
			}
			// language =================================
/*			String lang = prop.getProperty("lang");
			if (lang != null) {
				PeaProperties.setLanguagesBundle(lang);
			} else {
				PeaProperties.setLanguagesBundle(Locale.getDefault().getLanguage());
			} */
			// dimension and location ========================
			int w,h, x, y;			
			String frameX = prop.getProperty("frame_x");
			try {
				x = (int) Double.parseDouble(frameX);
			} catch (Exception e) {
				new UnexpectedValueException("frameX", "String", "is not digit: " + frameX).printDescription();
				x = PeaProperties.getFrameLocation().x;
			}			
			String frameY = prop.getProperty("frame_y");
			try {
				y = (int) Double.parseDouble(frameY);
			} catch (Exception e) {
				new UnexpectedValueException("frameY", "String", "is not digit: " + frameY).printDescription();
				y = PeaProperties.getFrameLocation().y;
			}			
			String frameWidth = prop.getProperty("frame_width");
			try {
				w = (int) Double.parseDouble(frameWidth);
			} catch (Exception e) {
				new UnexpectedValueException("frameWidth", "String", "is not digit: " + frameWidth).printDescription();
				w = PeaProperties.getFrameDim().width;
			}
			String frameHeight = prop.getProperty("frame_height");
			try {
				h = (int) Double.parseDouble(frameHeight);
			} catch (Exception e) {
				new UnexpectedValueException("frameHeight", "String", "is not digit: " + frameHeight).printDescription();
				h = PeaProperties.getFrameDim().height;
			}			
			PeaProperties.setFrameDim(new Dimension(w,h));
			PeaProperties.setFrameLocation(new Point(x,y));

		} catch (IOException e) {
			System.err.println("IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("Exception: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
			} catch (IOException e) {
				System.err.println("IOException: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}
	
	public String setSpecificPropValue() {
		String errorMessage = null;
		// There are no specific properties for Image Lock PEA and File Lock PEA:
		String fileType = PeaProperties.getFileType();
		if (fileType.equals("file")) { // File Lock PEA
			return null;
		} else if (fileType.equals("image")) { // Image Lock PEA
			return  null;			
		}
		try { 		
			
			//==========================================================
			// set PEA specific properties: 
			PeaControl peaControl = PeaControl.getDialog();
			
			Properties prop = new Properties();
			File file = new File(System.getProperty("user.dir") + File.separator + 
					"resources" + File.separator + 
					PeaControl.getPeaPropFileName());
			if (file.exists()) {
				// external file in resources
				inputStream = new FileInputStream(file);
			} else {
				// inside jar: default
				System.out.println("Load default pea specific properties...");
				inputStream = getClass().getResourceAsStream("/resources/" + PeaControl.getPeaPropFileName());
			}
			if (inputStream != null) {
				prop.load(inputStream);
			} else {
				new UnexpectedValueException("inputStream", "InputStream", "file not found: " + PeaControl.getPeaPropFileName()).printDescription();
				errorMessage = "file not found: " + PeaControl.getPeaPropFileName();
			}
			peaControl.setPeaPropValues(prop, inputStream);

/*			if (fileType.equals("text file")) { // Notebook PEA
				
			} else if (fileType.equals("file")) { // Notebook PEA
				
			} else if (fileType.equals("image")) { // Notebook PEA
				
			} else if (fileType.equals("calendar")) { // Notebook PEA
				CalendarSetting.setDefaultFileName(prop.getProperty("default_file_name"));
			} else {
				new UnexpectedValueException("fileType", "String", "does not match expected values: " + fileType).printDescription();
				errorMessage += "\nFileType does not match expected values: " + fileType;
			} */

		} catch (IOException e) {
			System.err.println("IOException: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();
		} catch (Exception e) {
			System.err.println("Exception: " + e.getMessage());
			errorMessage += e + ", " + e.getLocalizedMessage();			
		} finally {
			try {
				inputStream.close();
			} catch (Exception e) {
				System.err.println("Exception: " + e.getMessage());
				errorMessage += e + ", " + e.getLocalizedMessage();;
			}
		}
		return errorMessage;
	}


/*	public static String getPropfilenameXXX() {
		return propFileName;
	}*/

}
